/*******************************************************************************
* directory.cpp: programs definition and registration                          *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*******************************************************************************/


//------------------------------------------------------------------------------
// Preamble
//------------------------------------------------------------------------------
#include <stddef.h>
#include <stdio.h>
#include <pthread.h>

#include "config.h"
#include "common.h"
#include "string.h"
#include "list.h"
#include "log.h"
#include "directory.h"

#include "debug.h"
#include "list.cpp"



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_ES::C_ES(u16 iId, u8 iStreamType, const C_String& strName) : strDescr(strName)
{
  iPid = iId;
  iType = iStreamType;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_ES::~C_ES()
{
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Program::C_Program(const C_String& strLabel, u16 iPgrmId, u8 iState = PGRM_AVAILABLE) :
               strName(strLabel)
{
  iId = iPgrmId;
  iStatus = iState;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Program::~C_Program()
{
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
C_Program::C_Program(C_Program& cProgram)
{
  // Copy all datas that don't have copy constructor
  iPcrPid = cProgram.iPcrPid;
  iPsiPid = cProgram.iPsiPid;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_Program::AddES(C_ES* pEs)
{
  return cESList.PushEnd(pEs);
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
u16 C_Program::GetId() const
{
  return iId;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
u16 C_Program::GetPsiPid() const
{
  return iPsiPid;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
u8 C_Program::GetStatus() const
{
  return iStatus;
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
void C_Program::SetStatus(u8 iState)
{
  iStatus = iState;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Directory::C_Directory(handle hLogger)
{
  hLog = hLogger;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Directory::~C_Directory()
{
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_Directory::AddPgrm(C_Program* pProgram)
{
  ASSERT(pProgram);
  return cPgrms.PushEnd(pProgram);
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_Directory::RemovePgrm(unsigned int iIndex)
{
  int iRc = NO_ERR;
  
  if(iIndex >= cPgrms.Size())
  {
    Log(hLog, LOG_WARN, "Cannot remove program %d, it does not exist", iIndex);
    iRc = GEN_ERR;
  }
  else
    iRc = cPgrms.Remove(iIndex);

  return iRc;
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Program* C_Directory::GetPgrm(unsigned int iIndex) const
{
  C_Program* pPgrm = NULL;
  if(iIndex >= cPgrms.Size())
    Log(hLog, LOG_WARN, "Program %d does not exist", iIndex);
  else
    pPgrm = &cPgrms[iIndex];

  return pPgrm;
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
boolean C_Directory::IsAvailable(u16 iId)
{
  boolean bResult = 0;
  for(unsigned int iIndex = 0; iIndex < cPgrms.Size(); iIndex++)
  {
    if(cPgrms[iIndex].GetId() == iId)
    {
      bResult = 1;
      break;
    }
  }
  return bResult;
}

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
boolean C_Directory::IsBroadcasted(u16 iId)
{
  boolean bResult = 0;
  for(unsigned int iIndex = 0; iIndex < cPgrms.Size(); iIndex++)
  {
    if(cPgrms[iIndex].GetId() == iId &&
       cPgrms[iIndex].GetStatus() == PGRM_BROADCASTED)
    {
      bResult = 1;
      break;
    }
  }
  return bResult;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_Directory::EmptyPgrmList()
{
  for(unsigned int iIndex = 0; iIndex < cPgrms.Size(); iIndex++)
  {
    RemovePgrm(iIndex);
  }
  
  return 0;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
unsigned int C_Directory::GetPgrmNumber()
{
  return cPgrms.Size();
}
