/*******************************************************************************
* directory.h: Program description and registration classes definition         *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*------------------------------------------------------------------------------*
* required headers:                                                            *
*  "common.h"                                                                  *
*  "string.h"                                                                  *
*  "list.h"                                                                    *
*******************************************************************************/


#ifndef _DIRECTORY_H_
#define _DIRECTORY_H_


//------------------------------------------------------------------------------
// Defines
//------------------------------------------------------------------------------

// ES types as defined in ISO 13818-1
#define MPEG1_VIDEO_ES          0x01
#define MPEG2_VIDEO_ES          0x02
#define MPEG1_AUDIO_ES          0x03
#define MPEG2_AUDIO_ES          0x04

// Status of the program
#define PGRM_AVAILABLE        1
#define PGRM_BROADCASTED      2
#define PGRM_FINISHED         3
#define PGRM_ERROR            4
//#define PGRM_PAUSED           5
//#define PGRM_WAITING          6



//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
class C_ES
{
 public:
  C_ES(u16 iId, u8 iStreamType, const C_String& strDescr);
  ~C_ES();
  
 protected:
  // PID
  u16 iPid;
  // ISO 13818-1 type
  u8 iType;
  // Textual description if available
  C_String strDescr;
};



//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
class C_Program
{
 public:
  C_Program(const C_String& strName, u16 iPgrmNumber, u8 iState);
  ~C_Program();
  C_Program(C_Program& cProgram);
  
  int AddES(C_ES* pEs);
//  int RemoveEs(u16 iPid);
   
  // Read access to protected data member
  u16 GetId() const;
  u16 GetPsiPid() const;
  u8 GetStatus() const;
  void SetStatus(u8 iState);

 protected:
  // Program name, if available
  C_String strName;
  // List of associated elementary streams
  C_List<C_ES> cESList;
  // Program id
  u16 iId;
  // PID carrying the PCR for that program
  u16 iPcrPid;
  // PID of the PSI for that program
  u16 iPsiPid;
  // Status of the program
  u8 iStatus;
};


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
class C_Directory
{
 public:
  C_Directory(handle hLog);
  ~C_Directory();

  int AddPgrm(C_Program* pProgram);
  int RemovePgrm(unsigned int iIndex);
  C_Program* GetPgrm(unsigned int iIndex) const;

  boolean IsAvailable(u16 iId);
  boolean IsBroadcasted(u16 iId);

  int EmptyPgrmList();
  unsigned int GetPgrmNumber();

 protected:
  // Logging endpoint
  handle hLog;
  // List of programs available by the source
  C_List<C_Program> cPgrms;
};




#else
#error "Multiple inclusions of directory.h"
#endif
