/*******************************************************************************
* input.h: Input class definition                                              *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*------------------------------------------------------------------------------*
* required headers:                                                            *
*  "common.h"                                                                  *
*  "string.h"                                                                  *
*  "directory.h"                                                               *
*  "output.h"                                                                  *
*******************************************************************************/

#ifndef _INPUT_H_
#define _INPUT_H_


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
class C_Input
{
 public:
  C_Input();
  virtual ~C_Input();

  // Initialisation/termination
  int Init(const C_String& strCfgFile, const C_String& strSection);
  int Destroy();

  // Selection of the pgrm to broadcast. Programs are referenced by their index
  // in the program table
  int StartStreaming(unsigned int iIndex, const C_OutputCfg& cOutputCfg);
  int StopStreaming(unsigned int iIndex);

  // Child specific main loop
  virtual int Run() = 0;

 protected:
  // Child initialisation, in 3 steps. The InitParam method is called for every
  // param found in the configuration file
  virtual int PreInit() = 0;
  virtual int InitParam(const C_String& strName, const C_String& strVal) = 0;
  virtual int PostInit() = 0;

  // Child specific termination
  virtual int Clean() = 0;

  // Child program reception
  virtual int StartReception(C_Output* pOutput, C_Program* pPgrm) = 0;
  virtual int StopReception(C_Program* pPgrm) = 0;

  // Pgrm table management
  virtual int BuildPgrmTable() = 0;
  virtual int UpdatePgrmTable() = 0;

  // Outpout registration
  int Register(C_Output* pOutput);

  // Log endpoint
  handle hLog;
  // List of programs
  C_Directory* pDirectory;
  // Buffer of TS packets
  C_NetList cNetList;
  // List of outputs
  C_List<C_Output> cOutputList;
};





#else
#error "Multiple inclusions of input.h"
#endif
