/*******************************************************************************
* output.h: output stream classes definitions                                  *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*------------------------------------------------------------------------------*
* required headers:                                                            *
*  <netinet/in.h>                                                              *
*  <sys/uio.h>                                                                 *
*  "common.h"                                                                  *
*  "string.h"                                                                  *
*  "ts.h"                                                                      *
*  "buffer.h"                                                                  *
*******************************************************************************/


#ifndef _OUTPUT_H_
#define _OUTPUT_H_



//------------------------------------------------------------------------------
// Output types
//------------------------------------------------------------------------------
#define OUTPUT_NET      1
#define OUTPUT_FILE     2
#define OUTPUT_FIFO     3



//------------------------------------------------------------------------------
// C_OutputCfg class: properties of an output
//------------------------------------------------------------------------------
// Provide a global method to configure an output
//------------------------------------------------------------------------------
class C_OutputCfg
{
 public:
  C_OutputCfg(byte iType);
  ~C_OutputCfg();
  
  // Type of the output
  byte iType;

  // C_NetOutput configuration
  C_String strSrcHost;
  C_String strSrcPort;
  C_String strDstHost;
  C_String strDstPort;
};


//------------------------------------------------------------------------------
// C_Output class: generic output streaming
//------------------------------------------------------------------------------
// Provide a generic target for the output of data. Subclasses must be defined
// and used to send the data to specific device classes. 
//------------------------------------------------------------------------------
class C_Output
{
 public:
  C_Output(C_NetList& cNetList, handle hLogger, int iBuffSize);
  virtual ~C_Output();

  virtual int Init(const C_OutputCfg& cCfg) = 0;
  virtual int Destroy() = 0;

  int Send(C_TsPacket* pPacket);
  int Flush();

 protected:
  virtual int WriteToPort() = 0;

  C_Fifo* pBuff;
  C_NetList* pNetList;
  handle hLog;
};



//------------------------------------------------------------------------------
// C_NetOutput: output streaming over networks
//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
class C_NetOutput : public C_Output
{
 public:
  C_NetOutput(C_NetList& cNetList, handle hLogger);
  virtual ~C_NetOutput();

  virtual int Init(const C_OutputCfg& cCfg);
  virtual int Destroy();

 protected:
  virtual int WriteToPort();

  int BuildInetAddr(struct sockaddr_in* pInAddr, const C_String& strHost,
                    const C_String& strPort);

  int iSockFd;
  struct sockaddr_in saSrc;
  struct sockaddr_in saDst;

  // In order to decrease the overhead of sending multiple packets, we will
  // use writev to send the buffer with a single system call
  struct iovec* pBuffVector;
};



#else
#error "Multiple inclusions of output.h"
#endif
