/*******************************************************************************
* ts.cpp: TS packet manipulation                                               *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
* Code is awfull as regards Psi, it has to be rewritten                        *
*******************************************************************************/


//------------------------------------------------------------------------------
// Preamble
//------------------------------------------------------------------------------
#include <stdio.h>
#include <netinet/in.h>

#include "config.h"
#include "common.h"
#include "string.h"
#include "list.h"
#include "directory.h"
#include "ts.h"

#include "debug.h"



//******************************************************************************
// C_TsPacket class
//******************************************************************************
//
//******************************************************************************

//------------------------------------------------------------------------------
// Constructor
//------------------------------------------------------------------------------
C_TsPacket::C_TsPacket()
{
//  iTsId = 0;
//  iPgrmNumber = 0;
//  iPid = 0;
}



//------------------------------------------------------------------------------
// Constructor
//------------------------------------------------------------------------------
C_TsPacket::C_TsPacket(const byte* pPacket)
{
//  iTsId = 0;
//  iPgrmNumber = 0;
//  iPid = 0;

  // Store the raw buffer in the packet
  memcpy(bData, pPacket, TS_PACKET_LEN);
}


//------------------------------------------------------------------------------
// Destructor
//------------------------------------------------------------------------------
C_TsPacket::~C_TsPacket()
{
//  printf("TS %p destroyed (%p): %x %x %x %x %x %x %x %x %x %x %x\n", this,
//         this->bData, bData[0], bData[1], bData[2], bData[3], bData[4],
//         bData[5], bData[6], bData[7], bData[8], bData[9], bData[10]);
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
// No direct write access is provided so that we can be sure that the TS packet
// is checked before being stored within the class
//------------------------------------------------------------------------------
int C_TsPacket::Fill(const byte* pPacket)
{
  ASSERT(pPacket);

  int iRc = NO_ERR;

  /* Check sync byte */
  if(pPacket[0] != 0x47)
    iRc = SYNC_ERR;
  else
  {
//    iPid = U16_AT(bData[1]) & 0x1fff;
    memcpy(&bData, pPacket, TS_PACKET_LEN);
  }

  return iRc;
};


//C_TsPacket::Empty()
//{
//  i_TsId = 0;
//  iPgrmNumber = 0;
//  iPid = 0;
//}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// Give direct access to the raw buffer
//------------------------------------------------------------------------------
C_TsPacket::operator const byte* () const
{
  return bData;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// Direct byte manipulation
//------------------------------------------------------------------------------
byte& C_TsPacket:: operator [] (unsigned int iByte)
{
  return bData[iByte];
}



//------------------------------------------------------------------------------
// BuildHeader: Build a TS header and return its size
//------------------------------------------------------------------------------
// Beware: only the 13 first bits of the iPid field are taken into account, and
// only the 4 first bits of the iCounter are used
// No adaption field can yet be put in the header
// The method returns the length of the header
//------------------------------------------------------------------------------
int C_TsPacket::BuildHeader(u16 iPid, boolean bUnitStart, u8 iCounter)
{
  // For bUnitStart is a real boolean in the TS header
  ASSERT(bUnitStart == 0 || bUnitStart == 1);
  
  // Sync byte
  bData[0] = 0x47;
  // Set PID value and both error indicator and priority to 0
  SET_U16_TO(bData[1], iPid & 0x1FFF);
  // Payload unit start
  bData[1] |= bUnitStart << 6;
  // Continuity counter and no scrambling
  bData[3] = iCounter & 0x0F;
  // Adaption field control: always payload and never adaption field
  bData[3] |= 0x10;

  return 4;
}




//******************************************************************************
// C_PsiSection class
//******************************************************************************
// In the MPEG2 system norm, a section can be split into several TS packets, but
// as this is really unuseful, our implementation don't allow sections biggest
// than the size of the available payload of a TS packet.
//******************************************************************************

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_PsiSection::C_PsiSection()
{
  // Init the CRC encoder
  BuildCrc32Table();
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_PsiSection::~C_PsiSection()
{
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_PsiSection::BuildSection(u16 iPid, u8 iCounter, u8 iTableId,
                               u16 iPayloadId, u8 iVersion, boolean bCurrentNext,
                               u8 iCurrentSection, u8 iLastSection,
                               u16 iPayloadLength, const byte* pPayload)
{
  ASSERT(pPayload);
  ASSERT(bCurrentNext == 0 || bCurrentNext == 1);

  int iRc = NO_ERR;

  // First build the TS header: the unit_start indicator will be set to 1
  // indicating that a PSI section begins in the packet
  unsigned int iSize = C_TsPacket::BuildHeader(iPid, 1, iCounter);

  // Now add the pointer field immediatly after the header: as we don't know
  // its size yet, the payload will began immediatly after the pointer field
  // and its end will be marked by FF stuffing bytes
  bData[iSize] = 0;
  iSize++;

  // Table id
  bData[iSize] = iTableId;
  // Set section Section length is the one of the header yet
  SET_U16_TO(bData[iSize+1], iPayloadLength+9 & 0xFFF);

  // Section syntax set to one
  bData[iSize+1] |= 0x80;

  // Id
  SET_U16_TO(bData[iSize+3], iPayloadId);

  // Versioning
  bData[iSize+5] = (iVersion & 0x1F) << 1;
  bData[iSize+5] |= bCurrentNext;

  // Section management
  bData[iSize+6] = iCurrentSection;
  bData[iSize+7] = iLastSection;

  // Now we can start to fill in the payload  
  if(iPayloadLength+iSize+12 > TS_PACKET_LEN)
    iRc = GEN_ERR;
  else
  {
    // Fill in payload
    memcpy(&bData[iSize+8], pPayload, iPayloadLength);

    // Set the CRC field
    u32 iCrc = 0xffffffff;
    for(unsigned int i = iSize; i < iSize+iPayloadLength+8; i++)
      iCrc = (iCrc << 8) ^ iCrc32Table[(iCrc >> 24) ^ bData[i]];
    SET_U32_TO(bData[iSize+iPayloadLength+8], iCrc);
  }

  // Stuff the end of the packet with FF
  for(unsigned int iByte = iSize+iPayloadLength+12; iByte < TS_PACKET_LEN; iByte++)
    bData[iByte] = 0xFF;
  
  return iRc;
}



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
void C_PsiSection::BuildCrc32Table()
{
  u32 i, j, k;
  for( i = 0 ; i < 256 ; i++ )
  {
    k = 0;
    for (j = (i << 24) | 0x800000 ; j != 0x80000000 ; j <<= 1)
      k = (k << 1) ^ (((k ^ j) & 0x80000000) ? 0x04c11db7 : 0);
    iCrc32Table[i] = k;
  }
}








//******************************************************************************
// C_PatSection class
//******************************************************************************
// A PAT can contain the description of several program, but as we just want to
// build PAT for one program, this class has been restricted
//******************************************************************************

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
int C_PatSection::BuildSection(u8 iCounter, u16 iStreamId, u8 iVersion,
                               boolean bCurrentNext, u8 iCurrentSection,
                               u8 iLastSection, const C_Program& cPgrm)
{
  byte aBuff[4];
  
  SET_U16_TO(aBuff[0], cPgrm.GetId());
  SET_U16_TO(aBuff[2], cPgrm.GetPsiPid());

C_PsiSection::BuildSection(0, iCounter, 0, iStreamId, iVersion, bCurrentNext,
                           iCurrentSection, iLastSection, sizeof(aBuff), aBuff);
  
  return 0;
}
