/*******************************************************************************
* ts.h: TsPacket class definition                                              *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*------------------------------------------------------------------------------*
* required headers:                                                            *
*  "common.h"                                                                  *
*******************************************************************************/

#ifndef _TS_H_
#define _TS_H_


//------------------------------------------------------------------------------
// Defines
//------------------------------------------------------------------------------
#define TS_PACKET_LEN 188


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
class C_TsPacket
{
 public:
  C_TsPacket();
  C_TsPacket(const byte* pPacket);
  ~C_TsPacket();

  // Indirect buffer access
  int Fill(const byte* pPacket);  
  // Direct access to the buffer
  operator const byte* () const;
  
  // Direct byte manipulation
  byte& operator [] (unsigned int iByte);

  // By hand packet header construction
  int BuildHeader(u16 iPid, boolean bUnitStart, u8 iCounter);

 protected:
//  u16 iTsId;
//  u16 iPgrmNumber;
//  u16 iPid;
  
  byte bData[TS_PACKET_LEN];
};



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// At that time, the complete section must fit in a single TS packet. Moreover,
// if this class is used to build a private section, then the section will have
// the extension associated with a section_syntax_indicator set to 1
//------------------------------------------------------------------------------
class C_PsiSection : public C_TsPacket
{
 public:
  C_PsiSection();
  ~C_PsiSection();

  // Guess :)
  int BuildSection(u16 iPid, u8 iCounter, u8 iTableId, u16 iPayloadId,
                  u8 iVersion, boolean bCurrentNext, u8 iCurrentSection,
                  u8 iLastSection, u16 iPayloadLength, const byte* pPayload);

// protected:
  void BuildCrc32Table();
  u32 iCrc32Table[256];
};



//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
class C_PatSection : public C_PsiSection
{
 public:
//  C_PatSection();
//  ~C_PatSection();

  int BuildSection(u8 iCounter, u16 iStreamId, u8 iVersion, boolean bCurrentNext,
                   u8 iCurrentSection, u8 iLastSection, const C_Program& cPgrm);
};



#else
#error "Multiple inclusions of ts.h"
#endif
